# MailBot

This module will help you build an imap bot reacting to incoming emails.

You mainly provide two functions:

* A *trigger* which will be called for each received e-mail and should return a promise of a truthy if mail should trigger some job
* A *mailHandler* which will be called for each *triggered* e-mail and will do its job

This is usable but in active development, work in progress:

* Missing tests
* Missing more extensive README
* Add default options

## Installation

```sh
npm install --save mailbot
```

## Usage

```js
const { createBot } = require('mailbot')

const bot = createBot(options)

bot.start()
```

### API

```js
// Start watching, returns a Promise
bot.start()

// Stops watching, returns a Promise
// if argument is true, it will destroy the connection immediately
// you're strongly advised to gracefully disconnect by not setting this parameter
bot.stop(destroy = false)

// Restarts
bot.restart(destroy = false)

// Update configuration option
// Note: if you update 'imap', 'mailbox', or 'filter', it will restart the bot
// unless said otherwise
bot.configure('imap', connectionInfo, autoRestart = true, destroy = false)
```

### Options

```js
{

// IMAP configuration, see module "imap"
imap: {
	user: "user@gmail.com",
	password: "password",
	host: "imap.googlemail.com",
	port: 993,
	keepalive: true,
	tls: true,
	tlsOptions: {
		rejectUnauthorized: false
	},
},

// Watched inbox
mailbox: 'INBOX',

// Should bot mark fetched emails as read?
// If true, you're sure you will never fetch same mail twice even when restarting
// If false, you'll mess with server emails
markSeen: true,

// Search filter to fetch emails
filter: ['UNSEEN'],

// Should the trigger be checked when receiving headers, or when body has been parsed?
// If your control depends only on headers (subject, recipient, sender…), you can set it to true
// Warning: in 'headers' phase, headers are not parsed yet and you may need helpers
triggerOnHeaders: false,

// The trigger: this function is called for each e-mail
// Input: a mail object (if triggerOnHeaders is true, it will only have 'headers' property)
// Output: a value or a promise of a value
// The mail will be "handled" only if final value is truthy
trigger ({ headers }) {
	// Example: work with e-mails whose subject is 'BOT: <something>'
	const match = headers.subject.match(/BOT: (.*)$/)
	return match && match[1]
},

// The mail handler: this is the "job executor"
// Input: a mail object, and the trigger value
mailHandler (mail, trigger) {
	console.log({
		subject: mail.headers.subject,
		trigger
	})
},

// The error handler, called for each error occurring during processes
// As there may be error thrown from very different places,
// the function is called with a "context", which can be one of:
// - MAIL
// - INITIAL_SEARCH
// - INCREMENTAL_SEARCH
// - IMAP_ERROR
// - TRIGGER
errorHandler (error, context) {
	console.error(context, error)
},

// IMAP client reconnection
autoReconnect: true,
autoReconnectTimeout: 5000

}
```

### Mail objects

* Mail objects are generated by `mailparser` (version 0.x, not the buggy 2.0)

### Helpers

WIP: doc

* parseAddresses
* extractSignature

### Debugging

This module uses `debug` internally, and you can enable internal debug messages adding `mailbot` to your environment variable `DEBUG`:

```sh
env DEBUG=mailbot node mybot.js
```
